// NamedFolders.cpp : Defines the entry point for the DLL application.
// Far plugin "Named folders"
#include "stdafx.h"
#include <windows.h> 

#include <string.h>

#include "autobuf.h"
#include "common.h"
#include "registry.h"
#include "stdio.h"
#include <cassert>

#include "NFLang.h"
#include "namespanel.h"
#include "auxilary.h"

namespace 
{
	char const * PLUGIN_NAME = "Named Folders";
//default settings
	char const * DEFAULT_PREFIX = "cd:";
	
	const bool DEFAULT_SHOWERRORS = false;
	const bool DEFAULT_SHOWINPLUGINMENU = true;
	const bool DEFAULT_SHOWINDISKMENU = true;
	const bool DEFAULT_SHOWTEMPNAMESASHIDEFILES = true;
//	const bool DEFAULT_USEHIGHLIGHT = true;
	const bool DEFAULT_AUTOCOMPLITE = true;

	CAutoBufChar g_LastErrorName;	//        							
}

extern CAutoBufChar g_Prefix;			//

	void GetSettingsFromRegistry();
	void HandleError(terrors_t Error);
	terrors_t TreatCommand(char const* Command);	//   

///////////////////////////////////////////////////////////////////////////////////////
extern "C"
{
	void WINAPI _export SetStartupInfo(struct PluginStartupInfo const* Info);
	HANDLE WINAPI _export OpenPlugin(int OpenFrom,int Item);
	void WINAPI _export GetPluginInfo(struct PluginInfo* Info);
	void WINAPI _export GetOpenPluginInfo(HANDLE hPlugin, struct OpenPluginInfo *Info);
	int WINAPI _export GetFindData(HANDLE hPlugin, struct PluginPanelItem **pPanelItem, int *pItemsNumber, int OpMode);
	int WINAPI _export SetDirectory(HANDLE hPlugin,const char *Dir,int OpMode);
	void WINAPI _export ClosePlugin(HANDLE hPlugin);
	int WINAPI _export ProcessKey(HANDLE hPlugin,int Key,unsigned int ControlState);
//ver 1.4
	int WINAPI _export GetMinFarVersion(void);
	int WINAPI _export Configure(int ItemNumber);
	int WINAPI _export MakeDirectory (HANDLE hPlugin, char *Name, int OpMode);
};

///////////////////////////////////////////////////////////////////////////////////////
void WINAPI _export SetStartupInfo(struct PluginStartupInfo *pInfo)
{
	TRACE("SetStartupInfo");
	GetSettingsFromRegistry();
	::g_PluginInfo = *pInfo;
	::g_FSF = *pInfo->FSF;
}

void WINAPI _export GetPluginInfo(struct PluginInfo *pInfo)
{
	TRACE("GetPluginInfo");

	pInfo->DiskMenuStringsNumber = (g_ShowInDiskMenu) ? 1 : 0;
	pInfo->PluginMenuStringsNumber = (g_ShowInPluginMenu) ? 1 : 0;
	pInfo->PluginConfigStringsNumber = 1;

	static const char *DiskMenuStrings[1];
	static const char *PluginMenuStrings[1];
	static const char *PluginConfigStrings[1];
	DiskMenuStrings[0] = GetMsg(LANG_DISKMENUSTRING);
	PluginMenuStrings[0]= GetMsg(LANG_PLUGINMENUSTRING);
	PluginConfigStrings[0]= GetMsg(LANG_PLUGINSCONFIGSTRING);

	pInfo->DiskMenuStrings= DiskMenuStrings;
	pInfo->PluginMenuStrings= PluginMenuStrings;
	pInfo->PluginConfigStrings = PluginConfigStrings;

	pInfo->StructSize = sizeof(*pInfo); 
	pInfo->Flags = 0;	//PF_FULLCMDLINE;
	pInfo->CommandPrefix = (char*)g_Prefix;
}

HANDLE WINAPI _export OpenPlugin(int OpenFrom, int Item)
{
	TRACE("OpenPlugin");
	try {
		terrors_t Error = ERROR_NO;
		switch (OpenFrom)
		{
		case OPEN_COMMANDLINE: 
			{
				const char* pCmd = reinterpret_cast<char const*>(Item);
				
				if(! lstrlen(pCmd)) return new CNamesPanel(); // cd:
				Error = TreatCommand(pCmd);	
				if (ERROR_NO != Error) 
				{
					//   cd:filter_name
					//,       , 
					//    
					CAutoBufChar filter;
					if (GetFilter(pCmd, filter))
					{
						CNamesPanel *pNP = new CNamesPanel();
						pNP->SetFilterAndUpdate(filter);
						return pNP;
					}
					HandleError(Error);
				}
			}
		break;
		case OPEN_PLUGINSMENU:
		case OPEN_DISKMENU:
			return new CNamesPanel();
		break;
		}
	} catch (...)
	{
		HandleError(ERROR_UNKNOWN_EXCEPTION);
	}
	
	return INVALID_HANDLE_VALUE;
}

void WINAPI _export GetOpenPluginInfo(HANDLE hPlugin, struct OpenPluginInfo *Info)
{
	try {
		CNamesPanel *p = reinterpret_cast<CNamesPanel*>(hPlugin);
		p->GetOpenPluginInfo(Info);
	} catch (...)
	{
		HandleError(ERROR_UNKNOWN_EXCEPTION);
	}
}

int WINAPI _export GetFindData(HANDLE hPlugin, struct PluginPanelItem **pPanelItem, int *pItemsNumber, int OpMode)
{
	try {
		CNamesPanel *p = reinterpret_cast<CNamesPanel*>(hPlugin);
		return p->GetFindData(pPanelItem, pItemsNumber, OpMode);
	} catch (...)
	{
		HandleError(ERROR_UNKNOWN_EXCEPTION);
		return FALSE;
	}
}

int WINAPI _export SetDirectory(HANDLE hPlugin, const char *Dir, int OpMode)
{
	try {
		CNamesPanel *p = reinterpret_cast<CNamesPanel*>(hPlugin);
		return p->SetDirectory(Dir, OpMode);
	} catch (...)
	{
		HandleError(ERROR_UNKNOWN_EXCEPTION);
		return FALSE;
	}
}

void WINAPI ClosePlugin(HANDLE hPlugin)
{
	try {
		CNamesPanel *p = reinterpret_cast<CNamesPanel*>(hPlugin);
		delete p;
	} catch (...)
	{
		HandleError(ERROR_UNKNOWN_EXCEPTION);
	}
}

int WINAPI _export ProcessKey(HANDLE hPlugin,int Key,unsigned int ControlState)
{
	try {
		CNamesPanel *p = reinterpret_cast<CNamesPanel*>(hPlugin);
		return p->ProcessKey(Key, ControlState);
	} catch (...)
	{
		HandleError(ERROR_UNKNOWN_EXCEPTION);
		return FALSE;
	}
}

int WINAPI _export GetMinFarVersion(void)
{
	return MAKEFARVERSION(1,70,1282);
}

extern int ConfiguratePlugin();
int WINAPI _export Configure(int ItemNumber)
{
	try {
		return ConfiguratePlugin();
	} catch (...)
	{
		HandleError(ERROR_UNKNOWN_EXCEPTION);
		return FALSE;
	}
}


int WINAPI _export MakeDirectory (HANDLE hPlugin, char *Name, int OpMode)
{
	try {
		CNamesPanel *p = reinterpret_cast<CNamesPanel*>(hPlugin);
		return p->MakeDirectory(Name, OpMode);
	} catch (...)
	{
		HandleError(ERROR_UNKNOWN_EXCEPTION);
		return FALSE;
	}
}

///////////////////////////////////////////////////////////////////////////////////
terrors_t DecodeCommandLine(const char* CmdLine, CAutoBufChar &cmd, CAutoBufChar &path)
{
	TRACE("ExpandCommand");
/*
	CmdLine         <>[\|/][]
		"cd:pf\far",  ..
	   bu:
		command="pf", path = "far"
*/
	cmd.Assign(CmdLine);
//   
	char *pbuf = (char*)cmd;
	while (*pbuf != 0)
	{
		switch (*pbuf)
		{
		case '/': 
		case '\\':
			*pbuf = 0;
			path.Assign(pbuf + 1);
			break;
		default: ++pbuf;
		}
	}

	return ERROR_NO;
};

extern DWORD GetFolderAndTypeByIncompleteName(char const* Name, CAutoBufChar &buf);

terrors_t TreatCommand(char const* pCmd)
{	
	TRACE("ThreatCommand");
	/*
		     .
		: 
			:b -   b   .
			+b -    b   .  
				     .
			-b -   b   
			b -   ,    b
	*/

	try {
		switch (*pCmd)
		{
	//  
		case ':':
	//  
		case '+':
			{
				PanelInfo pni;
				g_PluginInfo.Control(INVALID_HANDLE_VALUE, FCTL_GETPANELINFO, &pni);
				return AddName(pCmd + 1, pni.CurDir, *pCmd == ':');
			}
		break;
	// 
		case '-': 
			return RemoveName(pCmd + 1, false); break;
		default:
	//    
			{
				CAutoBufChar cmd;
				CAutoBufChar path;
				terrors_t ErrorCode = DecodeCommandLine(pCmd, cmd, path);
				if (ErrorCode != ERROR_NO) return ErrorCode;

				CAutoBufChar buf;	//  ,   .
				DWORD Type = GetFolderAndType(cmd, buf);
				
			//   
				if ((Type != REG_SZ) && (g_AutoComplite)) //     
														  // 
					Type = GetFolderAndTypeByIncompleteName(cmd, buf);

				if (REG_SZ == Type)
				{
					//   		
					if (path.GetSize())
					{	//  
						buf += "\\";
						buf += (char*)path;
					}
					if (IsVirtualPath(buf)) 
						CloseAndStartOtherPlugin(buf, INVALID_HANDLE_VALUE, false);
					else 
						g_PluginInfo.Control(INVALID_HANDLE_VALUE, FCTL_CLOSEPLUGIN, (char*)buf);
				} else 			
				{
					if (g_ShowErrors)
							g_LastErrorName.Assign(pCmd);
					return ERROR_UNKNOWN_NAME;
				}
			}
		}
	} catch (terrors_t e)
	{
		return e;
	} catch (...)
	{
		return ERROR_UNKNOWN_EXCEPTION;
	}

	return ERROR_NO;
}


DWORD GetValueFromRegistry(CRegistry &reg, const char* Key, DWORD Default) throw (DWORD)
{
	DWORD value;
	if (0 == reg.GetValue(Key, value))	
	{
		reg.SetValue(Key, Default);	
		value = Default;
	}
	return value;
}

void GetSettingsFromRegistry()
{
	TRACE("GetSettingsFromRegistry");
//get prefix
	try {
		CRegistry reg(true);
		
		if (0 == reg.GetValue(KEY_PREFIX, g_Prefix))	
		{
			reg.SetValue(KEY_PREFIX, DEFAULT_PREFIX);	//  
			g_Prefix.Assign(DEFAULT_PREFIX);
		}

		g_ShowErrors = GetValueFromRegistry(reg, KEY_SHOWERRORS, DEFAULT_SHOWERRORS);
		g_ShowInPluginMenu = GetValueFromRegistry(reg, KEY_SHOWINPLUGINMENU, DEFAULT_SHOWINPLUGINMENU);
		g_ShowInDiskMenu = GetValueFromRegistry(reg, KEY_SHOWINDISKMENU, DEFAULT_SHOWINDISKMENU);
		g_ShowTemporaryNamesAsHideFiles = GetValueFromRegistry(reg, KEY_SHOWTEMPNAMESASHIDEFILES, DEFAULT_SHOWTEMPNAMESASHIDEFILES);
		// .   FAR     , 
		//   g_UseHiglighting  / 
		g_UseHighlighting = true;//GetValueFromRegistry(reg, KEY_USEHIGHLIGHT, DEFAULT_USEHIGHLIGHT);
		g_AutoComplite = GetValueFromRegistry(reg, KEY_AUTOCOMPLITE, DEFAULT_AUTOCOMPLITE);

	} catch (DWORD)
	{
		//        
		g_Prefix.Assign(DEFAULT_PREFIX);
	}
}


void HandleError(terrors_t Error)
{
	//    
	if (! g_ShowErrors) return;
	if (Error > NUM_ERRORS) return;

	assert(Error > ERROR_NO && Error < NUM_ERRORS);

	CAutoBufChar e;

	const char *Msg[2];
	Msg[0] = GetMsg(LANG_ERROR);
	if (Error == ERROR_UNKNOWN_NAME)
	{
		e.Assign(GetMsg(pItems[Error]));
		e += g_LastErrorName;
		Msg[1] = static_cast<char*>(e);
	} else Msg[1] = GetMsg(pItems[Error]);
	
	g_PluginInfo.Message(g_PluginInfo.ModuleNumber, 
		FMSG_MB_OK, 
		NULL, 
		Msg,
		2, 
		0);
}

// C:\Program Files\Far\Plugins\NamedFolders\