// NamesPanel.cpp: implementation of the CNamesPanel class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "NamesPanel.h"
#include "farkeys.hpp"
#include "NFLang.h"
#include "registry.h"
#include <cassert>

/*
#include "shlwapi.h"
#pragma comment(lib, "shlwapi.lib")
*/
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
extern terrors_t RemoveName(char const *pName, bool IsFilter);
extern DWORD GetFolderAndType(char const* Name, CAutoBufChar &buf);
extern terrors_t AddName(char const* pCmd, char const* pPath, bool bNonVolatile);
extern void CloseAndStartOtherPlugin(char const *cmd, HANDLE hPlugin, bool bInOtherPanel);
extern BOOL IsVirtualPath(char const *buf);


#define VK_R 52
namespace 
{
	void SetPanelItem(PluginPanelItem& item, char const* name, char * folder, bool bHidden, bool AsDirectory)
	{
		memset(&item, 0, sizeof(item));
		item.Description = folder;
		item.Flags = 0;
		lstrcpy(item.FindData.cFileName, name);
		item.FindData.dwFileAttributes =  (bHidden) ? FILE_ATTRIBUTE_HIDDEN | FILE_ATTRIBUTE_SYSTEM : 0;
		if (AsDirectory)
			item.FindData.dwFileAttributes |= FILE_ATTRIBUTE_DIRECTORY;
		item.Owner = NULL;
		item.CustomColumnData = NULL;
	}
}
CNamesPanel::CNamesPanel() throw (terrors_t) 
: m_NumPanelItems(0), m_pPanelItems(NULL), m_pFolders(NULL), m_AllocatedNumberItems(0)
{	
	SetFilter(0);
	UpdateListItems();
}

CNamesPanel::~CNamesPanel()
{
	DeleteListItems();
}

void CNamesPanel::DeleteListItems()
{
	alloc_free(m_pPanelItems);
	m_pPanelItems = NULL;
	for (UINT i = 0; i < m_NumPanelItems; ++i)
		delete m_pFolders[i];
	alloc_free(m_pFolders);
	m_pFolders = NULL;
	m_NumPanelItems = 0;
}

void CNamesPanel::AddItemsToList(const char *SubKey,	//    
								 bool bNonVolatile,		//   
								 bool bAsDirectory,		//      
								 char const* pExcludeFolder)	//     
{	//     
	try 
	{
		CRegistry reg(SubKey, bNonVolatile, false);
		CRegistryEnum er(reg);

		DWORD NumItems; //   
	
		er.GetInfo(&NumItems, NULL);
		m_AllocatedNumberItems += NumItems;

		//     
		//    ,   
		m_pPanelItems = reinterpret_cast<PluginPanelItem*>(realloc(m_pPanelItems, sizeof(PluginPanelItem) * m_AllocatedNumberItems));
		m_pFolders = reinterpret_cast<CAutoBufChar**>(realloc(m_pFolders, sizeof(CAutoBufChar*) * m_AllocatedNumberItems));

		CAutoBufChar buf;
		while (er.load_next_key(buf))
		{
		//  ,     
		// 
			if (m_NumPanelItems == m_AllocatedNumberItems)
			{
				realloc(m_pPanelItems, sizeof(PluginPanelItem) * (m_AllocatedNumberItems + 1));
				realloc(m_pFolders, sizeof(PluginPanelItem) * (m_AllocatedNumberItems + 1));
				++m_AllocatedNumberItems;
			}

		//   ,  
			m_pFolders[m_NumPanelItems] = new CAutoBufChar();

		//   
			if (IsNoFilter() || bAsDirectory)
			{
				reg.GetValue(buf, *m_pFolders[m_NumPanelItems]);
			} else 
			{  
				bool ret = true;
				reg.GetValue(buf, *m_pFolders[m_NumPanelItems]);
				while (! g_PluginInfo.CmpName(m_CurrentFilter, *m_pFolders[m_NumPanelItems], FALSE))
				{
					if (! er.load_next_key(buf))
					{
						ret = false;
						break;
					}
					reg.GetValue(buf, *m_pFolders[m_NumPanelItems]);
				}
				if (! ret)
				{
					delete m_pFolders[m_NumPanelItems];	//      
					break;
				}
	/*
				CAutoBufChar buf;
				buf.Alloc(30);
				::PathCompactPathEx(buf, *m_pFolders[m_NumPanelItems], 30, 0);
				m_pFolders[m_NumPanelItems]->Assign(buf);
	*/
			}
			if (pExcludeFolder != NULL)
				if (! lstrcmp(pExcludeFolder, static_cast<char*>(*m_pFolders[m_NumPanelItems]))) //    ( )
					continue;

		// ,  
		//     
			SetPanelItem(m_pPanelItems[m_NumPanelItems], buf, static_cast<char*>(*m_pFolders[m_NumPanelItems]), (g_ShowTemporaryNamesAsHideFiles) ? ! bNonVolatile : 0, bAsDirectory);
			++m_NumPanelItems;
		}
	} catch (terrors_t)
	{
		throw;
	} catch (...)
	{
		throw (ERROR_UNKNOWN_EXCEPTION);
	}
}

terrors_t CNamesPanel::UpdateListItems()
{
	//     
	//       	
	assert(0 == m_NumPanelItems);
	try 
	{
		char const *pFilter = m_CurrentFilter;
		AddItemsToList(REGISTRY_PLUGIN_FILTERS_SUBKEY, true, true, (char*)m_CurrentFilter);
		AddItemsToList(REGISTRY_PLUGIN_NON_VOLATILE_LISTKEYS_SUBKEY, true, false, NULL);
		AddItemsToList(REGISTRY_PLUGIN_VOLATILE_LISTKEYS_SUBKEY, false, false, NULL);
	} catch (terrors_t e)
	{
		return e;
	} 
	return ERROR_NO;
}

///////////////////////////////////////////////////////////////////////
int CNamesPanel::GetFindData(PluginPanelItem **pPanelItem, int *pItemsNumber, int OpMode)
{	
	static struct PanelInfo PInfo;
	if (! g_PluginInfo.Control(this, FCTL_GETPANELINFO, &PInfo)) 
		return FALSE;
	*pPanelItem = m_pPanelItems;
	*pItemsNumber = m_NumPanelItems;
	return TRUE;
}

void CNamesPanel::GetOpenPluginInfo(struct OpenPluginInfo *pi)
{
	pi->StructSize = sizeof(*pi);
	pi->Flags = OPIF_USEFILTER 
				| OPIF_ADDDOTS 
				| OPIF_SHOWNAMESONLY 
				| OPIF_SHOWRIGHTALIGNNAMES
				| OPIF_RAWSELECTION
				| OPIF_SHOWPRESERVECASE;
	if (g_UseHighlighting)
		pi->Flags |= OPIF_USEHIGHLIGHTING;
	pi->HostFile = NULL;
	pi->CurDir = "";

	pi->Format = (char*)GetMsg(LANG_NAMEDFOLDERS);

	pi->PanelTitle = static_cast<char*>(m_PanelTitle);

	pi->InfoLines = NULL;
	pi->InfoLinesNumber = 0;

	pi->DescrFiles = NULL;
	pi->DescrFilesNumber = 0;

	const int NUM_PANEL_MODES = 10;
	static struct PanelMode PanelModesArray[NUM_PANEL_MODES];
	PanelModesArray[6].ColumnTypes = NULL;
	PanelModesArray[6].ColumnWidths = "8,0";
	PanelModesArray[6].FullScreen = FALSE;
	PanelModesArray[6].Reserved[0] = 0;
	PanelModesArray[6].Reserved[1] = 0;

	pi->PanelModesArray = PanelModesArray;
	pi->PanelModesNumber = NUM_PANEL_MODES;
	pi->StartPanelMode='6';

	static struct KeyBarTitles KeyBar;
	memset(&KeyBar, 0, sizeof(KeyBar));
	KeyBar.Titles[3-1] = "";
	KeyBar.Titles[5-1] = "";
	KeyBar.Titles[6-1] = "";
	KeyBar.Titles[8-1] = (char*)GetMsg(LANG_F8);
	KeyBar.Titles[7-1] = (char*)GetMsg(LANG_F7);
	pi->KeyBar = &KeyBar;
}

int CNamesPanel::SetDirectory(const char *Dir, int OpMode)
{
	CAutoBufChar folder;
	if (! GetFolderAndType(Dir, folder)) 
		return FALSE;	//   

	if (IsVirtualPath(folder)) 
		CloseAndStartOtherPlugin(folder, this, false);
	else 
		g_PluginInfo.Control(this, FCTL_CLOSEPLUGIN, (void*)folder);

	return TRUE;
}

int CNamesPanel::SetDirectoryInOtherPanel(const char *Dir, int OpMode)
{
	CAutoBufChar folder;
	if (! GetFolderAndType(Dir, folder)) 
		return FALSE;	//   
	if (IsVirtualPath(folder)) 
		CloseAndStartOtherPlugin(folder, this, true);
//		return FALSE;	//        ...   :-)
	else 
	{
		g_PluginInfo.Control(this, FCTL_SETANOTHERPANELDIR, (void*)folder);
		g_PluginInfo.Control(this, FCTL_UPDATEANOTHERPANEL, 0);
		g_PluginInfo.Control(this, FCTL_REDRAWANOTHERPANEL, 0);
	}
	
	return TRUE;
}

int CNamesPanel::PutFiles(struct PluginPanelItem *PanelItem, int ItemsNumber, int Move, int OpMode)
{
	return 0;
}

void SET_FAR_DIALOG_ITEM(FarDialogItem &item, 
						 int Type,
						 int x1,
						 int y1,
						 int x2,
						 int y2,
						 int Focus,
						 char* History,
						 DWORD Flags,
						 int DefButtons,
						 char const *p)
{
	item.Type = Type;
	item.X1 = x1;
	item.Y1 = y1;
	item.X2 = x2;
	item.Y2 = y2;
	item.Focus = Focus;
	item.History = History;
	item.Flags = Flags;
	item.DefaultButton = DefButtons;
	lstrcpy(item.Data, p);
}


int CNamesPanel::ProcessKey(int Key, unsigned int ControlState)
{
	switch (Key)
	{
		case VK_RETURN:
		case VK_LBUTTON:	
		{ //    
			CAutoBufChar name;
			PanelInfo pi;

			//    
			if (g_PluginInfo.Control(this, FCTL_GETPANELINFO, &pi))
			{
			//   
				if (pi.SelectedItems[0].FindData.dwFileAttributes && FILE_ATTRIBUTE_DIRECTORY)
				{
					if (0 == pi.CurrentItem)
					{	//   ".."
						if (IsNoFilter())
						{	//      FAR  
							return FALSE;
						}
						//    " "
						SetFilter(0);
					}
					else //  
						SetFilter(pi.SelectedItems[0].Description);
					//   switch..
				} else {
				//   
					name.Assign(pi.SelectedItems[0].FindData.cFileName);
					if (ControlState == PKF_SHIFT)
					{	//  
						CAutoBufChar folder;
						CAutoBufChar folderqq;
						if (0 != GetFolderAndType(name, folder))
						{
							if (IsVirtualPath(folder)) return FALSE; //     
							folderqq.Alloc(folder.GetSize() + 3);
							folderqq.Assign("explorer \"");
							folderqq += folder;
							folderqq += "\"";
							STARTUPINFO si;
							memset(&si, 0, sizeof(STARTUPINFO));
							si.cb = sizeof(STARTUPINFO);
							PROCESS_INFORMATION pi;
							if (::CreateProcess(0, (char*)folderqq, NULL, NULL,
								0, 0, NULL, 0, &si, &pi))
								return TRUE;
						}
					} else if (ControlState == PKF_CONTROL)
				//   
						return SetDirectoryInOtherPanel(name, 0);
				//  FAR
					SetDirectory(name, 0);	
					return TRUE;
				}
			}
		} break;
		case VK_F8: 
		{
			if (0 != ControlState) return FALSE;

			try {
				//      
				PanelInfo pi;
				if (! g_PluginInfo.Control(this, FCTL_GETPANELINFO, &pi)) return FALSE;

				CAutoBufChar buf;
				if (1 == pi.SelectedItemsNumber) 
				{
					buf.Alloc(lstrlen(pi.SelectedItems[0].FindData.cFileName) +
						lstrlen(GetMsg(LANG_WARNING_REMOVE_SINGLE_NAME)) + 2);
					g_FSF.sprintf(static_cast<char*>(buf), GetMsg(LANG_WARNING_REMOVE_SINGLE_NAME), pi.SelectedItems[0].FindData.cFileName);
				} else 
				{
					buf.Alloc(lstrlen(GetMsg(LANG_WARNING_REMOVE_SEVERAL_NAMES)) + 10);
					g_FSF.sprintf(static_cast<char*>(buf), GetMsg(LANG_WARNING_REMOVE_SEVERAL_NAMES), pi.SelectedItemsNumber);
				}

				const char* Msgs[2] = {GetMsg(LANG_DELETE), static_cast<char*>(buf)};
				if (0 != g_PluginInfo.Message(g_PluginInfo.ModuleNumber, 
					FMSG_MB_OKCANCEL,
					NULL, 
					Msgs, 2, 0))  return FALSE;

				for (int i = 0; i < pi.SelectedItemsNumber; ++i)
					RemoveName(pi.SelectedItems[i].FindData.cFileName, pi.SelectedItems[i].FindData.dwFileAttributes && FILE_ATTRIBUTE_DIRECTORY);
			} catch (...)
			{
				return FALSE;
			}
		} break;
		case 'R':
			if (! (ControlState && PKF_CONTROL)) return FALSE;
			//Update below
		break;
		case VK_F4:
			{
				CAutoBufChar name;
				CAutoBufChar value;
				PanelInfo pi;

				bool bNew = (ControlState && PKF_SHIFT);
				bool bEdit = (0 == ControlState);

				if (! (bEdit || bNew))  return FALSE;

				//    
				if (g_PluginInfo.Control(this, FCTL_GETPANELINFO, &pi))
				{
					if (bEdit && (pi.SelectedItemsNumber == 0))
						return TRUE;		// ".."  				
					if (bEdit && (pi.SelectedItems[0].FindData.dwFileAttributes && FILE_ATTRIBUTE_DIRECTORY))
						return EditDirectory(pi.SelectedItems[0].FindData.cFileName, pi.SelectedItems[0].Description);
					name.Assign(pi.SelectedItems[0].FindData.cFileName);
				}
					
				const int NUM_ITEMS = 8;
				const int WIDTH = 50;
				const int HEIGHT = 10;
				//    
				static char *InsertHistoryName = "NamedFoldersInsertHistoryName";
				static char *InsertHistoryPath = "NamedFoldersInsertHistoryPath";
				FarDialogItem Items[NUM_ITEMS];

				int ni = 0;		//    Items
				const int FX = 2, FY = 1;		//     
				const int GAPX = 4, GAPY = 2;		//    

				if (bEdit) 
					GetFolderAndType(pi.SelectedItems[0].FindData.cFileName, value);
				else
				if (g_PluginInfo.Control(this, FCTL_GETANOTHERPANELINFO, &pi))
				{
					if (IsVirtualPath(pi.CurDir))
					{
						value.Assign(pi.CurDir);
					} else value.Assign(pi.CurDir);
				}

			// 
				SET_FAR_DIALOG_ITEM(Items[ni++], DI_DOUBLEBOX, FX, FY, WIDTH-FX-1, HEIGHT-FY-1, 0, 0, 
					0, 0, (bEdit) ? GetMsg(LANG_INS_TITLE_EDIT) : GetMsg(LANG_INS_TITLE_NEW));

				int nline = FY+1;	// ,     
			//:
				SET_FAR_DIALOG_ITEM(Items[ni++], DI_TEXT, GAPX, nline, 0, 0,  0, NULL, 
					0, 0, GetMsg(LANG_INS_NAME));
				nline += 1;
			// 
				const int idNAME = ni;
				SET_FAR_DIALOG_ITEM(Items[ni++], DI_EDIT, GAPX, nline, WIDTH-1-GAPX, 1, 1, InsertHistoryName, 
					(bEdit) ? DIF_HISTORY : DIF_HISTORY, 0, (bEdit) ? name : "");
				nline += 1;
			//:
				SET_FAR_DIALOG_ITEM(Items[ni++], DI_TEXT, GAPX, nline, 0, 0,  0, NULL, 
					0, 0, GetMsg(LANG_INS_PATH));
				nline += 1;
			// :
				const int idPATH = ni;
				SET_FAR_DIALOG_ITEM(Items[ni++], DI_EDIT, GAPX, nline, WIDTH-1-GAPX, 1,  1, InsertHistoryPath, 
					DIF_HISTORY, 0, value);
				nline += 1;
			//   
				const int idVOLATILE = ni;
				SET_FAR_DIALOG_ITEM(Items[ni++], DI_CHECKBOX, GAPX+1, nline, 0, 0, 0, NULL, 
					(bEdit) ? DIF_HIDDEN: 0, 0, GetMsg(LANG_INS_FLAG_TEMPORARY));
				nline += 1;
			// OK & Cancel
				const int idOK = ni;
				SET_FAR_DIALOG_ITEM(Items[ni++], DI_BUTTON, 16, nline, 0, 0,  0, NULL, 
					0, 0, "&OK" );
				const int idCancel = ni;
				SET_FAR_DIALOG_ITEM(Items[ni++], DI_BUTTON, 24, nline, 0, 0,  0, NULL, 
					0, 0, "&Cancel");

				int nChoosenItem = g_PluginInfo.Dialog(g_PluginInfo.ModuleNumber,-1, -1, WIDTH, HEIGHT, 
					NULL, Items, NUM_ITEMS);

				if (::lstrcmp(static_cast<char*>(name), Items[idNAME].Data))
				{	//  
					//:       ,  
					//.
					CAutoBufChar buf;
					if (GetFolderAndType(Items[idNAME].Data, buf))
					{
						const char* Msgs[2] = {GetMsg(LANG_ASK_OVERRIDE_NAME), static_cast<char*>(name)};
						if (0 != g_PluginInfo.Message(g_PluginInfo.ModuleNumber, 
							FMSG_MB_OKCANCEL,
							NULL, 
							Msgs, 2, 0)) 
							return TRUE;
					}
				}
				if ((nChoosenItem != -1) && (nChoosenItem != idCancel))
				{
					AddName(Items[idNAME].Data, Items[idPATH].Data, ! Items[idVOLATILE].Selected);
				} else return TRUE;		
			}
		break;
		case VK_F3: 
		case VK_F5: 
		case VK_F6: 
			return TRUE;	//   FAR'
		default:
			return FALSE;
	}; //switch

	FullUpdate();
	return TRUE;
}


void CNamesPanel::FullUpdate()
{
	//update panel
	DeleteListItems();
	UpdateListItems();

	g_PluginInfo.Control(this, FCTL_UPDATEPANEL, NULL);
	g_PluginInfo.Control(this, FCTL_REDRAWPANEL, NULL);
}

int CNamesPanel::MakeDirectory(char *Name, int OpMode)
{
	return EditDirectory("", "");
}

int CNamesPanel::EditDirectory(char const *FilterName, char const* Filter)
{
	//  
	const int WIDTH = 52;
	const int HEIGHT = 10;
	const int NUM_ITEMS = 7;
	FarDialogItem Items[NUM_ITEMS];

	int ni = 0;						//    Items
	const int FX = 2, FY = 1;		//     
	const int GAPX = 4, GAPY = 2;	//    


	// 
		SET_FAR_DIALOG_ITEM(Items[ni++], DI_DOUBLEBOX, FX, FY, WIDTH-FX-1, HEIGHT-FY-1, 0, 0, 
			0, 0, GetMsg(LANG_NEW_FILTER_TITLE));
		int nline = FY+1;		// ,     

		SET_FAR_DIALOG_ITEM(Items[ni++], DI_TEXT, GAPX+1, nline, 0, 0,  0, NULL, 
			0, 0, GetMsg(LANG_NEW_FILTER_NAME));
		nline += 1;
		const int idNAMEFILTER= ni;
		SET_FAR_DIALOG_ITEM(Items[ni++], DI_EDIT, GAPX+1, nline, WIDTH-GAPX-2, 1, 1, NULL, 
			0, 0, FilterName);
		nline += 1;
		SET_FAR_DIALOG_ITEM(Items[ni++], DI_TEXT, GAPX+1, nline, 0, 0,  0, NULL, 
			0, 0, GetMsg(LANG_NEW_FILTER_FILTER));
		nline += 1;
		const int idFILTER = ni;
		SET_FAR_DIALOG_ITEM(Items[ni++], DI_EDIT, GAPX+1, nline, WIDTH-GAPX-2, 1, 1, NULL, 
			0, 0, Filter);
		nline += 2;
	// OK & Cancel
		const int idOK = ni;
		SET_FAR_DIALOG_ITEM(Items[ni++], DI_BUTTON, 16, nline, 0, 0,  0, NULL, 
			0, 0, "&OK" );
		const int idCancel = ni;
		SET_FAR_DIALOG_ITEM(Items[ni++], DI_BUTTON, 24, nline, 0, 0,  0, NULL, 
			0, 0, "&Cancel");

	int nChoosenItem = g_PluginInfo.Dialog(
		g_PluginInfo.ModuleNumber, -1, -1, 
		WIDTH, HEIGHT, NULL, Items, NUM_ITEMS);
	if ((nChoosenItem == -1) || (nChoosenItem == idCancel))
		return TRUE;

	CRegistry reg(REGISTRY_PLUGIN_FILTERS_SUBKEY, true, true);
	reg.SetValue(Items[idNAMEFILTER].Data, Items[idFILTER].Data);
	
	FullUpdate();
	return TRUE;
}

void CNamesPanel::SetFilter(char const *sFilter)
{
	static char Title[30];
	g_FSF.sprintf(Title," %s",GetMsg(LANG_NAMEDFOLDERS));
	m_PanelTitle.Assign(Title);
	if (NULL == sFilter)
	{
		m_CurrentFilter.Assign("");
	} else {
		m_PanelTitle += ": ";
		m_PanelTitle += sFilter;
		m_CurrentFilter.Assign(sFilter);
	}
}
