// Registry.cpp: implementation of the CRegistry class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Registry.h"
#include <cassert>
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
extern char const * REGISTRY_PLUGIN_DATA_KEY;


CRegistry::CRegistry(bool bNonVolatile, bool bCreateIFNotExisted) throw (DWORD)
{
	if (ERROR_SUCCESS != RegCreateKeyEx(HKEY_CURRENT_USER, 
		REGISTRY_PLUGIN_DATA_KEY, 0, NULL, 
		(bNonVolatile) ? REG_OPTION_NON_VOLATILE : REG_OPTION_VOLATILE,
		KEY_ALL_ACCESS, NULL, &m_hKey, NULL))
			throw ERROR_REGISTRY;
}

CRegistry::CRegistry(const char* pszSubKey, bool bNonVolatile, bool bCreateIFNotExisted) throw (DWORD)
{
	CAutoBufChar buf(REGISTRY_PLUGIN_DATA_KEY);
	buf += pszSubKey;
	if (ERROR_SUCCESS != RegCreateKeyEx(HKEY_CURRENT_USER, 
		buf, 0, NULL, (bNonVolatile) ? REG_OPTION_NON_VOLATILE : REG_OPTION_VOLATILE,
		KEY_ALL_ACCESS, NULL, &m_hKey, NULL)) 
		throw ERROR_REGISTRY;
}

CRegistry::~CRegistry()
{
	UINT ErrorCode = RegCloseKey(m_hKey);
}


bool CRegistry::SetValue(const char* pKey, const char* pPath)
{
	return RegSetValueEx(m_hKey, pKey, 0, REG_SZ, (LPBYTE)pPath, strlen(pPath) + 1) == ERROR_SUCCESS;
}

DWORD CRegistry::GetValue(const char* pKey, CAutoBufChar &folder)
{
	DWORD size = MAX_PATH;	//    
	DWORD type;
	DWORD ErrorCode; 

	do {
		folder.Alloc(size);
		ErrorCode = RegQueryValueEx(m_hKey, pKey, 0, &type, (LPBYTE)&folder, &size);
	} while (ErrorCode == ERROR_MORE_DATA);

	if (! ((ErrorCode == ERROR_SUCCESS) && (type == REG_SZ))) return 0;
	return type;
}

bool CRegistry::Remove(const char* pKey)
{
	return RegDeleteValue(m_hKey, pKey) == ERROR_SUCCESS;
}

bool CRegistry::SetValue(char const* pKey, DWORD Value)
{
	return RegSetValueEx(m_hKey, pKey, 0, REG_DWORD, (LPBYTE)&Value,	 sizeof(Value)) == ERROR_SUCCESS;
}

bool CRegistry::GetValue(char const* pKey, DWORD &Value)
{
	DWORD type;
	DWORD size = sizeof(Value);
	bool b = (ERROR_SUCCESS == RegQueryValueEx(m_hKey, pKey, 0, &type, (LPBYTE)&Value, &size));
	return (type == REG_DWORD) && b;
}

///////////////////////////////////////////////////////////////////////////////////////
//
CRegistryEnum::CRegistryEnum(HKEY key) : m_hKey(key)
{
	m_dwIndex = 0;
}

bool CRegistryEnum::load_next_key(CAutoBufChar &Key)
{
	DWORD size = MAX_PATH; 

   // Loop until failure
	DWORD ErrorCode;
	do {
		Key.Alloc(size);
		ErrorCode = RegEnumValue(m_hKey, m_dwIndex, (LPSTR)&Key, 
			&size, NULL, NULL, NULL, NULL);
	} while (ErrorCode == ERROR_MORE_DATA);
	if (ErrorCode == ERROR_SUCCESS) 
	{
		m_dwIndex++;
		return true;
	}
	return false;
}


bool CRegistryEnum::GetInfo(DWORD *lpcValues, DWORD *lpcMaxValueNameLen) const
{
	return ERROR_SUCCESS == RegQueryInfoKey(m_hKey, NULL, NULL, NULL, NULL, NULL, 
		NULL, lpcValues, lpcMaxValueNameLen, NULL, NULL, NULL);
}