// auxilary.cpp: implementation of the auxilary class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "auxilary.h"

//////////////////////////////////////////////////////////////////////
extern CAutoBufChar g_Prefix;			//
//////////////////////////////////////////////////////////////////////

terrors_t RemoveName(char const *Name, bool IsFilter)
{	//    
	try {
		if (! IsFilter)
		{	// 
			CRegistry reg(REGISTRY_PLUGIN_NON_VOLATILE_LISTKEYS_SUBKEY, true);
			if (! reg.Remove(Name))	
			{
				CRegistry reg(REGISTRY_PLUGIN_VOLATILE_LISTKEYS_SUBKEY, false);
				if (! reg.Remove(Name))
					return ERROR_UNKNOWN_NAME;
			}
		} else 
		{	// 
			CRegistry reg(REGISTRY_PLUGIN_FILTERS_SUBKEY, true);
				if (! reg.Remove(Name))
					return ERROR_UNKNOWN_NAME;
		}

	} catch (terrors_t e)
	{
		return e;
	} catch (...)
	{
		return ERROR_UNKNOWN_EXCEPTION;
	}
	return ERROR_NO;
}


DWORD GetFolderAndType(char const* Name, CAutoBufChar &buf)
{	//      
	DWORD type = 0;
	try {	//    
		CRegistry reg(REGISTRY_PLUGIN_VOLATILE_LISTKEYS_SUBKEY, false);
		type = reg.GetValue(Name, buf);	//   
	} catch (DWORD)
	{
		//   
	}

	try {
		if (0 == type)
		{	//  ,    
			CRegistry reg(REGISTRY_PLUGIN_NON_VOLATILE_LISTKEYS_SUBKEY, true);
			type = reg.GetValue(Name, buf);	//   
		}
	} catch (DWORD)
	{
		//   
		return 0;
	}

	return type;
}


terrors_t AddName(char const* pCmd, char const* pPath, bool bNonVolatile)
{	
	/*
		     .
		 bVolatile = false,       
		     . 
		     .
	*/
	if (0 == *pCmd) return ERROR_WRONG_NAME_SYNTAX;
	try {
		CRegistry reg(
			(bNonVolatile) ? 
				REGISTRY_PLUGIN_NON_VOLATILE_LISTKEYS_SUBKEY : 
				REGISTRY_PLUGIN_VOLATILE_LISTKEYS_SUBKEY,
			bNonVolatile);

		UINT LenPrefix = IsVirtualPath(pCmd);
		if (LenPrefix)
		{	//   ":" (. "reg::x")
			//   "x"   "reg:folder"
			CAutoBufChar buf;
			buf.Alloc(LenPrefix + lstrlen(pPath) + 1);
			UINT i = 0;
			for (i = 0; i < LenPrefix; ++i)
				buf[i] = *(pCmd + i);
			buf[i] = 0;
			buf += pPath;	//  buf     
			reg.SetValue(pCmd + LenPrefix + 1, buf);	//+1 ..  
		} else reg.SetValue(pCmd, pPath);	
	} catch (terrors_t e)
	{
		return e;
	} catch (...)
	{
		return ERROR_UNKNOWN_EXCEPTION;
	}
	return ERROR_NO;
}


void CloseAndStartOtherPlugin(char const * cmd, HANDLE hPlugin, bool bInOtherPanel = false)
{	//""   cmd + Enter
	const UINT numberkeys = lstrlen(cmd);
	const UINT prefixsize = lstrlen(g_Prefix);

	static KeySequence ks;
	ks.Flags = 0;
	ks.Count = (numberkeys + 1);
	if (bInOtherPanel)	//     
		ks.Count += 3 + prefixsize; //TAB + TAB + "cd:" + ENTER 
	
	DWORD *pKeys = reinterpret_cast<DWORD*>(alloc_malloc(sizeof(DWORD) * ks.Count));
	ks.Sequence = pKeys;

	try 
	{
		if (bInOtherPanel)
		{
//			return;
			UINT j = 0, i;
			pKeys[j++] = VK_TAB;
			for (i = 0; i < numberkeys; ++i, ++j)
				pKeys[j] = static_cast<DWORD>(cmd[i]);
			pKeys[j++] = VK_RETURN;
			pKeys[j++] = VK_TAB;
			for (i = 0; i < prefixsize; ++i, ++j)
				pKeys[j] = static_cast<DWORD>(g_Prefix[i]);
			pKeys[j++] = VK_RETURN;
		} else {
			for (int i = 0; i < ks.Count - 1; ++i)
				pKeys[i] = static_cast<DWORD>(cmd[i]);
			pKeys[i] = VK_RETURN;
		}

		g_PluginInfo.AdvControl(g_PluginInfo.ModuleNumber, ACTL_POSTKEYSEQUENCE, reinterpret_cast<void*>(&ks));
	} catch(...)
	{
		//
	}
	alloc_free(pKeys);
}


bool IsNetworkCommand(CAutoBufChar &buf)
{
	if (buf.GetSize() > 2)
		return buf[0] == '\\' && buf[1] == '\\';
	return false;
}

BOOL IsVirtualPath(char const *buf)
{	//true  buf -  , i.e. buf = "reg:HKEY_CURRENT_USER\Software\Far"
	//false  buf   ":"         ":", i.e. "far", "c:"
	//     
	UINT size = lstrlen(buf);
	if (size  > 2)
	{
		UINT i = 0;
		while (i < size)
		{
			if (buf[i] == ':') 
				if ((i > 2) && (i != size)) 
				{
					//       ( ).
					char const *p = buf + size - 1;
					while (*p != ':') --p;
					return p - buf;
				}
			++i;
		}
	}
	return false;
}

bool GetFilter(char const* filtername, CAutoBufChar &buf)
{
	CRegistry reg(REGISTRY_PLUGIN_FILTERS_SUBKEY, false);
	return reg.GetValue(filtername, buf) == REG_SZ;	
}