// far_plug.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include "far_plug.h"
#include "farcolor.hpp"
#include "libgfl.h"
#include <objbase.h>
#include <assert.h>

#include "plugin.hpp"

static PluginStartupInfo Info;
static HWND FarWindow = 0;
static WORD FillColorPanel = 0;
static WORD FillColorView = 0;
static char** Exts = 0;
static DWORD ExtsNum = 0;

class CPicture
{
private:
    CHAR_INFO* buffer;
    CHAR_INFO* chi;
public:
    char FileName[1000];
    RECT ConsoleRect;
    RECT DCRect;
    RECT RangedRect;
    bool Redraw;
    bool ChangeName;
    WORD FillColor;
    GFL_BITMAP* RawPicture;
    unsigned char* DibData;
    BITMAPINFOHEADER BmpHeader;

    CPicture();
    ~CPicture();
    bool UpdateConsoleRectAndFileName();
    bool CheckName();
    void UpdateDCRect();
    void FillConsole();
    bool CheckForConsoleRedrawed();
    bool GeneratePicture();
    RECT RangingPic();
    bool Draw();
};

CPicture::CPicture()
{
    FileName[0] = 0;
    ConsoleRect.left = ConsoleRect.right = ConsoleRect.top = ConsoleRect.bottom = 0;
    DCRect.left = DCRect.right = DCRect.top = DCRect.bottom = 0;
    RangedRect.left = RangedRect.right = RangedRect.top = RangedRect.bottom = 0;
    Redraw = ChangeName = true;
    FillColor = 0;
    RawPicture = 0;
    DibData = 0;

    buffer = 0;
    chi = 0;
}

CPicture::~CPicture()
{
    if( DibData )
        free( DibData );
    if( RawPicture )
        gflFreeBitmap( RawPicture );
    RawPicture = 0;
    DibData = 0;
    if( buffer )
        free( buffer );
    buffer = 0;
    if( chi )
        free( chi );
    chi = 0;
}

static void GetDIBFromBitmap( GFL_BITMAP *bitmap, BITMAPINFOHEADER * bitmap_info,
                              unsigned char **data )
{
    int i, j, bytes_per_line; 
    unsigned char *ptr_src, *ptr_dst; 

	*data = NULL; 

	memset( bitmap_info, 0, sizeof(BITMAPINFOHEADER) ); 
	
	bitmap_info->biSize   = sizeof(BITMAPINFOHEADER); 
	bitmap_info->biWidth  = bitmap->Width; 
	bitmap_info->biHeight = bitmap->Height; 
	bitmap_info->biPlanes = 1; 
	
	bytes_per_line = (bitmap->Width * 3 + 3) & -4; 
	bitmap_info->biClrUsed = 0;
	bitmap_info->biBitCount = 24; 
    bitmap_info->biCompression = BI_RGB; 
    bitmap_info->biSizeImage = bytes_per_line * bitmap->Height; 
    bitmap_info->biClrImportant = 0; 
	
	*data = (unsigned char*)malloc( bitmap_info->biSizeImage ); 

	if ( bitmap->Type != GFL_BINARY
		&& bitmap->Type != GFL_GREY
		&& bitmap->Type != GFL_COLORS
        && bytes_per_line == (int)bitmap->BytesPerLine )
    {
        memcpy( *data, bitmap->Data, bitmap_info->biSizeImage );
		return;
    }
	
	for ( i = 0; i < bitmap->Height; i++ )
	{
		ptr_src = bitmap->Data + i * bitmap->BytesPerLine; 
		ptr_dst = *data + i * bytes_per_line; 

		if ( bitmap->Type == GFL_BINARY )
		{
			for ( j=0; j<bitmap->Width; j++ )
			{
                ptr_dst[0] = ptr_dst[1] = ptr_dst[2] = *ptr_src ? 255 : 0;
				ptr_dst += 3;
                ptr_src++;
			}
		}
        else if ( bitmap->Type == GFL_GREY )
		{
			for ( j=0; j<bitmap->Width; j++ )
			{
				*ptr_dst++ = *ptr_src; 
				*ptr_dst++ = *ptr_src; 
				*ptr_dst++ = *ptr_src++; 
			}
		}
		else if ( bitmap->Type == GFL_COLORS )
		{
			for ( j=0; j<bitmap->Width; j++ )
			{
				*ptr_dst++ = bitmap->ColorMap->Blue[ *ptr_src ]; 
				*ptr_dst++ = bitmap->ColorMap->Green[ *ptr_src ]; 
				*ptr_dst++ = bitmap->ColorMap->Red[ *ptr_src++ ]; 
			}
		}
        else if( bitmap->Type == GFL_RGB )
		{
			for ( j=0; j<bitmap->Width; j++ )
			{
				*ptr_dst++ = ptr_src[0]; 
				*ptr_dst++ = ptr_src[1]; 
				*ptr_dst++ = ptr_src[2]; 
				ptr_src += bitmap->BytesPerPixel; 
			}
		}
        else
		{
			for ( j=0; j<bitmap->Width; j++ )
			{
				*ptr_dst++ = ptr_src[2]; 
				*ptr_dst++ = ptr_src[1]; 
				*ptr_dst++ = ptr_src[0]; 
				ptr_src += bitmap->BytesPerPixel; 
			}
		}
	}
}

static WindowInfo GetActiveFarWindow()
{
    int num = Info.AdvControl( Info.ModuleNumber, ACTL_GETWINDOWCOUNT, 0 );
    WindowInfo info;
    info.Pos = -1;
    Info.AdvControl( Info.ModuleNumber, ACTL_GETWINDOWINFO, (void*)&info );

    return info;
}


#define QVIEW_STR "Quick view"
#define QVIEW_LEN 10

#define BORDER_V     0xba
#define BOTTOM_LEFT  0xc8
#define BOTTOM_RIGHT 0xbc

typedef BOOL IsActive_t();
static IsActive_t* IsActive =
  (IsActive_t*)GetProcAddress( GetModuleHandle( "infis.dll" ), "IsActive" );


bool CPicture::UpdateConsoleRectAndFileName()
{
    if(Info.AdvControl(Info.ModuleNumber,ACTL_CONSOLEMODE,(void*)CONSOLE_GET_MODE) == CONSOLE_FULLSCREEN ||
       IsActive && !IsActive() )
        return false;

    HANDLE out = GetStdHandle( STD_OUTPUT_HANDLE );
    CONSOLE_SCREEN_BUFFER_INFO screen;
    GetConsoleScreenBufferInfo( out, &screen );
    RECT res = {0, 0, 0, 0};

    WindowInfo winfo = GetActiveFarWindow();
    if( winfo.Type == WTYPE_VIEWER )
    {
        RECT rect = { 0, 1, screen.dwSize.X - 1, screen.dwSize.Y - 2 };
        if( strcmp(FileName, winfo.Name) )
        {
            Redraw = ChangeName = true;
            strcpy( FileName, winfo.Name );
            ConsoleRect = rect;
            UpdateDCRect();
        }
        else if( rect.left != ConsoleRect.left || rect.right != ConsoleRect.right ||
                rect.top != ConsoleRect.top || rect.bottom != ConsoleRect.bottom )
        {
            Redraw = true;
            ConsoleRect = rect;
            UpdateDCRect();
        }
        FillColor = FillColorView;
        return true;
    }
    else if( winfo.Type == WTYPE_PANELS )
    {
        FillColor = FillColorPanel;

        buffer = (CHAR_INFO*)realloc( buffer, screen.dwSize.X * screen.dwSize.Y * sizeof(*buffer) );
        COORD rcoord = {0, 0};
        DWORD read = 0;
        RECT qv = {0, 0, 0, 0};

        ReadConsoleOutputCharacter( out, (char*)buffer, screen.dwSize.X,
                                    rcoord, &read );
        // find string "Quick view"
        char* qvptr = (char*)buffer;
        int i = 0;
        for( i = 0; i < screen.dwSize.X - QVIEW_LEN &&
             memcmp( qvptr, QVIEW_STR, QVIEW_LEN ); i++, qvptr++ );
        if( i < screen.dwSize.X - QVIEW_LEN )
        {   // found string
            qv.top = 1;

            SMALL_RECT r = {0, 0, screen.dwSize.X, screen.dwSize.Y};
            ReadConsoleOutput( out, buffer, screen.dwSize, rcoord, &r );
            
            if( i + 5 < screen.dwSize.X / 2 )
            {   // left panel
                qv.left = 1;

                // find bottom left corner
                CHAR_INFO* lbc = buffer + screen.dwSize.X;
                int j;
                for( j = 1; j < screen.dwSize.Y && lbc->Char.AsciiChar != (char)BOTTOM_LEFT;
                     j++, lbc += screen.dwSize.X );
                if( j < screen.dwSize.Y )
                {   // found bottom left corner
                    qv.bottom = j - 3;
                    //find bottom right corner (to control)
                    for( j = 0; j < screen.dwSize.X &&
                                lbc->Char.AsciiChar != (char)BOTTOM_RIGHT;
                                j++, lbc++ );
                    if( j < screen.dwSize.X ) // ok, we found boorder
                        qv.right = j - 1;
                    else
                    {
                        Redraw = true;
                        return false;
                    }
                }
                else
                {
                    Redraw = true;
                    return false;
                }
            }
            else
            {   //right panel
                qv.right = screen.dwSize.X - 2;

                // find bottom right corner
                CHAR_INFO* lbc = buffer + screen.dwSize.X * 2 - 1;
                int j;
                for( j = 1; j < screen.dwSize.Y && lbc->Char.AsciiChar != (char)BOTTOM_RIGHT;
                     j++, lbc += screen.dwSize.X );
                if( j < screen.dwSize.Y )
                {   // found bottom right corner
                    qv.bottom = j - 3;
                    //find bottom left corner (to control)
                    for( j = qv.right + 1; j >= 0 &&
                                lbc->Char.AsciiChar != (char)BOTTOM_LEFT;
                                j--, lbc-- );
                    if( j >= 0 ) // ok, we found boorder
                        qv.left = j + 1;
                    else
                    {
                        Redraw = true;
                        return false;
                    }
                }
                else
                {
                    Redraw = true;
                    return false;
                }
            }
            assert( (unsigned char)buffer[qv.left - 1 + screen.dwSize.X].Char.AsciiChar == BORDER_V );
            assert( (unsigned char)buffer[qv.right + 1 + screen.dwSize.X].Char.AsciiChar == BORDER_V );
            assert( (unsigned char)buffer[qv.left - 1 + screen.dwSize.X * (qv.bottom + 3)].Char.AsciiChar == BOTTOM_LEFT );
            assert( (unsigned char)buffer[qv.right + 1 + screen.dwSize.X * (qv.bottom + 3)].Char.AsciiChar == BOTTOM_RIGHT );

            // getting file name
            char dir[1000];
            GetCurrentDirectory( 1000, dir );
            strcat( dir, "\\" );
            int position = (qv.bottom + 2) * screen.dwSize.X;
            for( i = qv.left; i <= qv.right && buffer[position + i].Char.AsciiChar; i++ )
            {
                char s[] = {buffer[position + i].Char.AsciiChar, 0};
                strcat( dir, s );
            }
            for( i = strlen(dir) - 1; i > 0 && dir[i] <= ' ';
                 dir[i] = 0, i-- );

            if( stricmp(FileName, dir) )
            {
                Redraw = ChangeName = true;
                strcpy( FileName, dir );
                ConsoleRect = qv;
                UpdateDCRect();
            }
            else if( qv.left != ConsoleRect.left || qv.right != ConsoleRect.right ||
                     qv.top != ConsoleRect.top || qv.bottom != ConsoleRect.bottom )
            {
                Redraw = true;
                ConsoleRect = qv;
                UpdateDCRect();
            }
            return true;
        }
        Redraw = true;
        return false;
    }
    Redraw = true;
    return false;
}

void CPicture::UpdateDCRect()
{
    RECT rect;
    CONSOLE_SCREEN_BUFFER_INFO info;

    GetWindowRect( FarWindow, &rect );
    GetConsoleScreenBufferInfo( GetStdHandle( STD_OUTPUT_HANDLE ), &info );

    int dx = (rect.right - rect.left) / info.dwSize.X;
    int dy = (rect.bottom - rect.top) / info.dwSize.Y;

    DCRect.left   = dx * ConsoleRect.left;
    DCRect.right  = dx * (ConsoleRect.right + 1);
    DCRect.top    = dy * ConsoleRect.top;
    DCRect.bottom = dy * (ConsoleRect.bottom + 1);
}

#define FILL_CONSTANT 1

void CPicture::FillConsole()
{
    COORD size = {ConsoleRect.right - ConsoleRect.left + 1,
                  ConsoleRect.bottom - ConsoleRect.top + 1};
    COORD start = {0, 0};
    SMALL_RECT rect = {(short)ConsoleRect.left, (short)ConsoleRect.top,
                       (short)ConsoleRect.right, (short)ConsoleRect.bottom};

    chi = (CHAR_INFO*)realloc( chi, size.X * size.Y * sizeof(*chi) );
    CHAR_INFO templ;
    templ.Attributes = FillColor;
    templ.Char.UnicodeChar = FILL_CONSTANT;
    for( int i = 0; i < size.X * size.Y; i++ )
        chi[i] = templ;
    WriteConsoleOutput( GetStdHandle( STD_OUTPUT_HANDLE ), chi, size, start, &rect );
    Redraw = false;
}

bool CPicture::CheckForConsoleRedrawed()
{
    COORD size = {ConsoleRect.right - ConsoleRect.left + 1,
                  ConsoleRect.bottom - ConsoleRect.top + 1};
    COORD start = {0, 0};
    SMALL_RECT rect = {(short)ConsoleRect.left, (short)ConsoleRect.top,
                       (short)ConsoleRect.right, (short)ConsoleRect.bottom};

    chi = (CHAR_INFO*)realloc( chi, size.X * size.Y * sizeof(*chi) );

    ReadConsoleOutput( GetStdHandle( STD_OUTPUT_HANDLE ), chi, size, start, &rect );
    for( int i = 0; i < size.X * size.Y; i++ )
        if( chi[i].Attributes != FillColor || chi[i].Char.UnicodeChar != FILL_CONSTANT )
            return true;

    return false;
}

bool CPicture::GeneratePicture()
{
    if( ChangeName )
    {
        if( DibData )
            free( DibData );
        if( RawPicture )
            gflFreeBitmap( RawPicture );
        RawPicture = 0;
        DibData = 0;

        Redraw = true;
        ChangeName = false;

        GFL_LOAD_PARAMS load_params; 
        gflGetDefaultLoadParams( &load_params ); 
        load_params.Flags |= GFL_LOAD_SKIP_ALPHA; 
        load_params.Origin = GFL_BOTTOM_LEFT; 
        load_params.ColorModel = GFL_BGR; 
        load_params.LinePadding = 4; 
        GFL_ERROR res = gflLoadBitmap( FileName, &RawPicture, &load_params, NULL );
        if( res )
        {
            RawPicture = 0;
            return false;
        }
    }

    if( Redraw && DibData )
    {
        free( DibData );
        DibData = 0;
    }

    if( !RawPicture )
        return false;

    if( Redraw )
    {
        GFL_BITMAP* pic = 0;
        RangedRect = RangingPic();
        gflResize( RawPicture, &pic, RangedRect.right, RangedRect.bottom, 0, 0 );
        if( !pic )
            return false;
        GetDIBFromBitmap( pic, &BmpHeader, &DibData); 
        gflFreeBitmap( pic );
    }

    return true;
}

RECT CPicture::RangingPic()
{
    float asp_dst = (float)(DCRect.right - DCRect.left) / (float)(DCRect.bottom - DCRect.top);
    float asp_src = (float)RawPicture->Width / (float)RawPicture->Height;

    int dst_w;
    int dst_h;
    
    if( asp_dst < asp_src )
    {
        dst_w = min( DCRect.right - DCRect.left, RawPicture->Width );
        dst_h = (int)(dst_w / asp_src);
    }
    else
    {
        dst_h = min( DCRect.bottom - DCRect.top, RawPicture->Height );
        dst_w = (int)(asp_src * dst_h);
    }

    RECT dest = {DCRect.left, DCRect.top, dst_w, dst_h};
    return dest;
}

bool CPicture::Draw()
{
    if( !DibData )
        return false;

    if( Redraw || CheckForConsoleRedrawed() )
        FillConsole();

    HDC hDC; 
    hDC = GetDC( FarWindow );

    StretchDIBits( hDC, 
        RangedRect.left, RangedRect.top,
        RangedRect.right, RangedRect.bottom, 
        0, 0, RangedRect.right, RangedRect.bottom,
        DibData,
        (BITMAPINFO *)&BmpHeader, DIB_RGB_COLORS, SRCCOPY ); 

    ReleaseDC( FarWindow, hDC );
    return true;
}

bool CPicture::CheckName()
{
    DWORD flen = strlen( FileName );
    for( DWORD i = 0; i < ExtsNum && Exts; i++ )
        if( Exts[i] && strlen( Exts[i] ) < flen )
            if( !stricmp( FileName + flen - strlen( Exts[i] ), Exts[i] ) &&
                FileName[flen - strlen( Exts[i] ) - 1] == '.' )
                return true;
    return false;
}

static int Run = 1;
CRITICAL_SECTION crit;

static DWORD DELAY_T = 300;

DWORD WINAPI ThreadProc( LPVOID lpParameter )
{
    GFL_BITMAP *TheBitmap = NULL; 

    SendMessage( FarWindow, WM_PAINT, 0, 0 );

    CPicture pic;

    while( Run )
    {
        EnterCriticalSection( &crit );
            if( pic.UpdateConsoleRectAndFileName() )
                if( pic.CheckName() )
                    if( pic.GeneratePicture() )
                        pic.Draw();
        LeaveCriticalSection( &crit );
        Sleep( DELAY_T );
    }
    return 0;
}


static HANDLE thread;
BOOL WINAPI DllMain( HINSTANCE, DWORD fdwReason, LPVOID )
{
    switch( fdwReason )
    {
    case DLL_PROCESS_ATTACH:
        InitializeCriticalSection( &crit );
        memset( &Info, 0, sizeof( Info ) );
        break;
    case DLL_PROCESS_DETACH:
        Run = 0;
        EnterCriticalSection( &crit );
        TerminateThread( thread, 0 );
        WaitForSingleObject( thread, 1000 );
        break;
    }
    return TRUE;
}

void WINAPI ClosePlugin( HANDLE plugin )
{
    return;
}

void WINAPI GetPluginInfo( struct PluginInfo* info )
{
    static char* Menus[] = { "Show pictures" };

    info->StructSize = sizeof( *info );
    info->CommandPrefix = "show";
    info->PluginMenuStrings = Menus;
    info->PluginMenuStringsNumber = sizeof( Menus ) /
                                    sizeof( *(info->PluginMenuStrings) );
    info->PluginConfigStrings = Menus;
    info->PluginConfigStringsNumber = sizeof( Menus ) /
                                      sizeof( *(info->PluginMenuStrings) );
    info->Flags = PF_PRELOAD;
    return;
}

#define KEY_NAME "\\Pic"

static bool GetParam( const char* keyname, char* value, DWORD size, const char* def )
{
    char plug_key[1000];
    strcpy( plug_key, Info.RootKey );
    strcat( plug_key, KEY_NAME );
    HKEY key = 0;

    DWORD type;

    if( RegOpenKey( HKEY_CURRENT_USER, plug_key, &key ) != ERROR_SUCCESS ||
        RegQueryValueEx( key, keyname, 0, &type,
                         (unsigned char*)value, &size ) != ERROR_SUCCESS )
    {
        if( def )
            strcpy( value, def );
        if( key )
            RegCloseKey( key );
        return false;
    }
    RegCloseKey( key );
    return true;
}

static bool GetParam( const char* keyname, DWORD* value, DWORD def )
{
    char plug_key[1000];
    strcpy( plug_key, Info.RootKey );
    strcat( plug_key, KEY_NAME );
    HKEY key = 0;

    DWORD type;
    DWORD size = sizeof( *value );

    if( RegOpenKey( HKEY_CURRENT_USER, plug_key, &key ) != ERROR_SUCCESS ||
        RegQueryValueEx( key, keyname, 0, &type,
                         (unsigned char*)value, &size ) != ERROR_SUCCESS )
    {
        *value = def;
        if( key )
            RegCloseKey( key );
        return false;
    }
    RegCloseKey( key );
    return true;
}

static void SetParam( const char* keyname, const char* value )
{
    char plug_key[1000];
    strcpy( plug_key, Info.RootKey );
    strcat( plug_key, KEY_NAME );
    HKEY key ;

    if( RegCreateKey( HKEY_CURRENT_USER, plug_key, &key ) != ERROR_SUCCESS )
        return;
    RegSetValueEx( key, keyname, 0, REG_SZ, (unsigned char*)value, strlen( value ) );
    RegCloseKey( key );
}

static void SetParam( const char* keyname, DWORD value )
{
    char plug_key[1000];
    strcpy( plug_key, Info.RootKey );
    strcat( plug_key, KEY_NAME );
    HKEY key ;

    if( RegCreateKey( HKEY_CURRENT_USER, plug_key, &key ) != ERROR_SUCCESS )
        return;
    RegSetValueEx( key, keyname, 0, REG_DWORD, (unsigned char*)&value, sizeof( value ) );
    RegCloseKey( key );
}

static void SetDefaultExtentions()
{
    if( Exts )
    {
        for( DWORD i = 0; i < ExtsNum; i++ )
            if( Exts[i] )
                free( Exts[i] );
        Exts = 0;
    }

    int number = gflGetNumberOfFormat();

    GFL_FORMAT_INFORMATION finfo;

    for( int i = 0; i < number; i++ )
    {
        gflGetFormatInformationByIndex( i, &finfo );
        for( DWORD j = 0; j < finfo.NumberOfExtension; j++ )
        {
            Exts = (char**)realloc( Exts, (ExtsNum + 1) * sizeof( *Exts ) );
            Exts[ExtsNum] = strdup( finfo.Extension[j] );
            ExtsNum++;
        }
    }
}

static BOOL CALLBACK EnumWindowsProc(
  HWND hwnd,      // handle to parent window
  LPARAM lParam ) // application-defined value

{
    char* title = (char*)lParam;
    char buffer[1000];
    GetWindowText( hwnd, buffer, 1000 );
    if( !strstr( buffer, title ) )
        return TRUE;
    FarWindow = hwnd;
    return FALSE;
}

static int run = 1;
void WINAPI SetStartupInfo( struct PluginStartupInfo *info )
{
    Info = *info;
    DWORD id;

    DWORD far_version;
    Info.AdvControl( Info.ModuleNumber, ACTL_GETFARVERSION, &far_version );
    if( (far_version & 0xffff) < 0x0146  ||
        (far_version & 0xffff) == 0x0146 && (far_version & 0xffff0000) < 0x04760000 )
        return;

    char data[1000];

//    GetParam( "Extentions", data, 999, "PNG;JPEG;BMP;TARGA;TIF;IFF;GIF;PCX;XPM;PICT" );
//    ParseExtentions( data );

    GetParam( "LoadDefault", data, 4, "no" );
    GetParam( "SleepTime", &DELAY_T, 300 );

    if( stricmp( data, "yes" ) )
    {
        EnterCriticalSection( &crit );
        run = 0;
    }

    FillColorPanel = Info.AdvControl( Info.ModuleNumber, ACTL_GETCOLOR, (void*)COL_PANELTEXT );
    FillColorView = Info.AdvControl( Info.ModuleNumber, ACTL_GETCOLOR, (void*)COL_VIEWERTEXT );
    FillColorPanel = (FillColorPanel & 0xf0) + ((FillColorPanel & 0xf0) >> 4);
    FillColorView = (FillColorView & 0xf0) + ((FillColorView & 0xf0) >> 4);

    OSVERSIONINFO version;
    version.dwOSVersionInfoSize = sizeof( version );
    GetVersionEx( &version );
    if( version.dwPlatformId == VER_PLATFORM_WIN32_NT )
    {
        GUID guid;
        CoCreateGuid( &guid );
        char buffer[100];
        char title[1000];
        GetConsoleTitle( title, 1000 );
        sprintf( buffer, "%x-%x-%x-%x%x%x%x%x%x%x%x", guid.Data1, guid.Data2, guid.Data3,
                                                      guid.Data4[0], guid.Data4[1],
                                                      guid.Data4[2], guid.Data4[3],
                                                      guid.Data4[4], guid.Data4[5],
                                                      guid.Data4[6], guid.Data4[7] );
        SetConsoleTitle( buffer );
        EnumWindows( EnumWindowsProc, (long)buffer );
//        FarWindow = (HWND)Info.AdvControl( Info.ModuleNumber, ACTL_GETFARHWND, 0 );
    }
    else
        FarWindow = FindWindowEx( FindWindow( "tty", 0 ), NULL, "ttyGrab", NULL );

    thread = CreateThread( 0, 0, ThreadProc, 0, 0, &id );

    gflLibraryInit(); 
    gflEnableLZW( GFL_TRUE ); 

    SetDefaultExtentions();
}

HANDLE WINAPI OpenPlugin( int OpenFrom, int Item )
{
    if( run )
        EnterCriticalSection( &crit );
    else
        LeaveCriticalSection( &crit );

    run ^= 1;

    return INVALID_HANDLE_VALUE;
}

void WINAPI ExitFAR( void )
{
    Run = 0;
    EnterCriticalSection( &crit );
    TerminateThread( thread, 0 );
    WaitForSingleObject( thread, 1000 );
}

int WINAPI Configure( int ItemNumber )
{
    char mask[] = "99999";
    char hist[] = "pic_hist";

    char data[10];

    GetParam( "LoadDefault", data, 4, "no" );

    FarDialogItem items []={
        {DI_DOUBLEBOX, 3, 1, 51, 6, FALSE, 0, 0, 0, "Show pictures"},
        {DI_CHECKBOX, 5, 2, 0, 0, FALSE, 0, 0, 0, "&Load by default"},
        {DI_TEXT, 5, 3, 0, 0, 0, 0, 0, 0, "&Sleep time (ms) "},
        {DI_FIXEDIT, 5 + strlen( items[2].Data ), 3, 9 + strlen( items[2].Data ), 0, FALSE, (int)mask, DIF_MASKEDIT, 0, ""},
        {DI_TEXT, 0, 4, 0, 0, 0, 0, DIF_SEPARATOR, 0, 0},
        {DI_BUTTON, 0, 5, 0, 0, TRUE, TRUE, DIF_CENTERGROUP, TRUE, "&Ok"},
        {DI_BUTTON, 0, 5, 0, 0, TRUE, FALSE, DIF_CENTERGROUP, 0, "&Cancel"}
    };

    sprintf( items[3].Data, "%d", DELAY_T );

    if( !stricmp( data, "yes" ) )
        items[1].Selected = 1;

    int item_num = sizeof( items ) / sizeof( items[0] );
    int dialog_res = Info.Dialog( Info.ModuleNumber, -1, -1, 55, 8, 0,
                                  items, item_num );
    if( dialog_res == -1 || dialog_res == 6 )
        return 0;

    SetParam( "LoadDefault", items[1].Selected ? "yes" : "no" );
    sscanf( items[3].Data, "%d", &DELAY_T );
    SetParam( "SleepTime", DELAY_T );

    return 0;
}
